/* FILE: filter.dsp             Copyright (c), Symmetric Research, 1996
 *
 * This program demonstrates computing a simple two term filter on
 * values taken from the DSPA64, saving both the raw and filtered data
 * continuously in a single circular buffer in multiplexed fashion.
 *
 * The low pass filtering used here increases the effective resolution
 * of the system by signal averaging, demonstrating the ability of DSP to
 * perform real time numerical processing.  Of course the price of this
 * increased resolution is a decrease in bandwidth, but oversampling is
 * more than a reasonable trade off for many applications.
 *
 * The simple recursive filter used here is:
 *
 *
 *                     f[n] = SCALE*x[n] + C*f[n-1]
 *
 *
 * where x[n] is a raw input value, and f[n] is the filterd result.
 * When expanded out, you can see that the filtered values are:
 *
 *
 *     f[n] = SCALE*( x[n] + C*x[n-1] + (C^2)*x[n-2] + (C^3)*x[n-3] + ... )
 *
 *
 * The idea is to pick the coefficient C to be less than 1.0, and then a
 * weighted tailing off average of previous values is accumulated as the
 * filtered result.  This type of filtering is smoother than a simple
 * unweighted average carried back over only a finite number of terms.
 *
 * The coefficients of the two term filter are set by the in memory
 * variables SCALE and COEFF below.  To compute the filter, the incoming
 * values are converted to floating point, the filter computed, and then
 * the result saved to memory as a 16 bit integer for easy comparison
 * with the raw value.
 *
 * Of course much more sophisticated filters can be used.  Publications
 * from AT&T, and authors such as Oppenheim and Schafer cover details of
 * FIR and IIR filtering in great detail.  When implementing these more
 * complicated filters keep in mind that they must finish computing
 * before the next DSPA64 value is ready or else data will be lost.
 * Usually the time available is at least 72 or more ticks of the
 * DSPA64 clock.
 *
 * Data for this program is taken from a single multiplexer channel.
 * See some of the other programs in this directory for programs
 * involving channel switching.
 *
 * Finally, note that the filter coefficients could be changed in real
 * time while the system is executing.  The ability to change system
 * characteristics with only program modifications can be a very useful
 * feature of DSP.
 *
 *
 *
 *
 * To run this program download and execute filter.dso by hand in the
 * DSPMON monitor.  Display the acquired VALUES array with the dw
 * command.  The raw data will be in the column on the left, and the
 * filtered data in the column on the right.  Successive d commands
 * issued while you are executing will show newly acquired values.
 *
 *
 *
 *
 *
 */






/* ASSEMBLY CONSTANTS:
 *
 *
 *
 */

constant

        /* Program memory locations. */

        RESET           = 0x000000,
        DATA            = 0x001000,


        /* Channel selection. */

        CHANNEL         = 0x00;         /* 0x00 <= CHAN <= 0x3F */





<RESET>

main {

        register

                waits           = r1,

                channel         = r2,
                value           = r3,
                zero            = r4,
                coeff           = r5,
                scale           = r6;




        /* Program the DSP32C for 0 wait states. */

        waits = 0; pcw = waits;



        /* Initialize the serial port. */

        ioc = ILEN_16 | IN_MSB | OLEN_08 | OUT_MSB;


        

        /* Set the multiplexer channel. */

        channel = CHANNEL;
           obuf = channel.b;





        /* Read and filter 16 bit values from the DSPA64:
         *
         * Both the raw and filterd values are saved to memory as a circular
         * buffer.  Use the DSPMON dw command to display the data as two
         * columns.
         *
         *
         *
         *
         *
         */

         value = VALUES;

         scale = SCALE;
         coeff = COEFF;

          zero = ZERO;
            a0 = *zero;  /* initialize a0 */


  poll:

        if ( ibf ) {


                /* Read in and save the raw 16 bit DSPA64 value. */

                *value++ = ibuf.w;



                /* Compute a two term recursive filter:
                 *
                 * The filter computed is,
                 *
                 *      a0 = SCALE*a1 + COEFF*a0
                 *
                 *
                 * where the new filter value is accumulated and kept in
                 * a0, and the raw input value is kept in a1.
                 *
                 *
                 *
                 *
                 *
                 */

                a1 = float( ibuf);

                        nop;
                        nop;
                        nop;

                  a1 = *scale * a1;

                        nop;

                    a0 = a1 + *coeff * a0;
                    
                        nop;



                /* Save the filtered value as a 16 bit integer. */

                *value++ = a2 = int( a0);





                /* Check for the end of the array. */

                if ( value >= END_OF_VALUES )

                        value = VALUES;  /* reset back to beginning */


                }


        goto poll;


        nop;
        nop;
        nop;


}





/* FILTER COEFFICIENTS:
 *
 * The SCALE and COEFF coefficients set the filter characteristics, and
 * should be chosen to provide the desired filter response.
 *
 * 
 * 
 * CHOOSING SCALE ...
 * 
 * Selecting the SCALE coefficient is usually done so that that if the
 * input is 1, then the output will also be 1.  From the formulas above,
 * for a constant input of 1, the filtered values are:
 * 
 * 
 *       f[n] = SCALE*( 1 + C + C^2 + C^3 + ... ) = SCALE * 1/(1-C)
 * 
 * 
 * Where the infinite series in C is a well known sum.  Taking SCALE to
 * be (1-C) will return normalized filter values.
 *
 *
 *
 * 
 *
 * CHOOSING COEFF ...
 *
 * The C coefficient COEFF, should be chosen to be a number less than
 * 1.0 whose nth power is sufficiently small so the filter tail dies off
 * at the rate you would like.  Since C^n is the weight at the nth term,
 * choosing:
 *
 * 
 *            COEFF = (desired weight at nth term)^(1/n)
 *
 * 
 * will give an estimate of the value of COEFF.
 * 
 * 
 *
 * Note that while the two term filter is easy to implement and is
 * totally appropriate for many applications, the shape of the filter
 * tail is fixed.  More complicated filters involving more terms will
 * have filter tails that can be more fully specified.
 *
 *
 *
 *
 *
 *
 *
 *
 */

dspf

        SCALE           = 0.2000000,    /* <- should be (1-COFF) */
        COEFF           = 0.8000000,

        ZERO            = 0.0000000;











/* STORAGE AREA FOR INCOMING VALUES FROM THE DSPA64:
 *
 * Note that both the raw and filtered values are saved in the VALUES
 * array in multipexed fashion.
 *
 *
 */

constant NSAMPLES = 0x800;

<DATA>

word
        VALUES[NSAMPLES], END_OF_VALUES;











/* SERIAL PORT IOC CONTROL REGISTER BIT ASSIGMENTS:
 *
 * These constants define the bit assigments for the serial port IOC
 * configuration control register.  They provide convenient mnemonic names
 * for the bits.
 *
 *
 */

constant

        ILEN_00         = 0x0 << 0x06,          /* input length         */
        ILEN_08         = 0x1 << 0x06, 
        ILEN_16         = 0x2 << 0x06,
        ILEN_32         = 0x3 << 0x06,

        OLEN_00         = 0x0 << 0x0A,          /* output length        */
        OLEN_08         = 0x1 << 0x0A,
        OLEN_16         = 0x2 << 0x0A,
        OLEN_32         = 0x3 << 0x0A,

        IN_LSB          = 0x0 << 0x10,          /* input lst first      */
        IN_MSB          = 0x1 << 0x10,          /* input msb first      */

        OUT_LSB         = 0x0 << 0x11,          /* output lsb first     */
        OUT_MSB         = 0x1 << 0x11;          /* output msb first     */

