/* FILE: sample.dsp             Copyright (c), Symmetric Research, 1996
 *
 * This program demonstrates reading 16 bit values from the DSPA64
 * board, saving them continuousely as integers in a circular buffer.
 *
 * It has the minimum amount of code required, using simple polling to
 * read values from a single DSPA64 channel.  No numerical processing is
 * done on incoming values.
 *
 * A C program to augment this could be as simple as doing get_32C's on
 * the VALUES array to pass them on to the PC.  See some of the other
 * programs in this directory for additional programming examples.
 *
 *
 * To run this program, download and execute sample.dso by hand in the
 * DSPMON monitor.  Examine the VALUES array before, during, and after
 * execution to see the data values.  Experiment with putting different
 * voltages on the sampled channel to see the resulting values.
 *
 *
 *
 */



/* ASSEMBLY CONSTANTS:
 *
 * These assembly time constants specify the program memory locations, and
 * channel selection.
 *
 *
 */

constant

        /* Program memory locations. */

        RESET           = 0x000000,
        DATA            = 0x000800,


        /* Channel selection. */

        CHANNEL         = 0x00;         /* 0x00 <= CHANNEL <= 0x3F */






<RESET>

main {
        register

                waits           = r1,

                channel         = r2,
                value           = r3;




        /* Program the DSP32C for 0 wait states. */

        waits = 0; pcw = waits;



        /* Initialize the serial port. */

        ioc = ILEN_16 | IN_MSB | OLEN_08 | OUT_MSB;




        /* Set the multiplexer channel:
         *
         * Note that constants cannot be loaded directly into obuf.  The
         * DSP32C instruction set only allows obuf to be loaded from a CAU
         * rN register memory.
         *
         *
         */

        channel = CHANNEL;
           obuf = channel.b;





        /* Read 16 bit integer values from the DSPA64:
         *
         * This loop polls the DSP32C serial input buffer ibuf, saving the
         * most significant 16 bits of acquired DSPA64 values to memory.
         *
         * The incoming data is stored in a circular buffer, where the input
         * pointer is reset to the beginning of the values array whenever the
         * end of the array is reached.
         *
         * No processing is done on incoming values.  They are simply saved
         * to memory.  Typical additional tasks might be converting integer
         * values to floating point, computing a digital filter, etc.  See
         * some of the other programs in this directory for examples.
         *
         *
         *
         */

        value = VALUES;

  poll:

        if ( ibf ) {


                /* Save the data value. */

                *value++ = ibuf.w;



                /* Check for the end of the array. */

                if ( value >= END_OF_VALUES )

                        value = VALUES;  /* reset back to beginning */

                }


        goto poll;

}








/* STORGE AREA FOR INCOMING DATA VALUES FROM THE DSPA64:
 *
 * The VALUES array is where incoming data is stored.  END_OF_VALUES marks
 * the end of the array.
 *
 * You may increase the size of the VALUES array until available memory
 * on your DSP card is exhausted.
 *
 *
 */

constant NVALUES = 0x100;

<DATA>

word
        VALUES[NVALUES] = { 0 }, END_OF_VALUES;











/* SERIAL PORT IOC CONTROL REGISTER BIT ASSIGMENTS:
 *
 * These constants define the bit assigments for the serial port IOC
 * configuration control register.  They provide convenient mnemonic names
 * for the bits.
 *
 *
 */

constant

        ILEN_00         = 0x0 << 0x06,          /* input length         */
        ILEN_08         = 0x1 << 0x06, 
        ILEN_16         = 0x2 << 0x06,
        ILEN_32         = 0x3 << 0x06,

        OLEN_00         = 0x0 << 0x0A,          /* output length        */
        OLEN_08         = 0x1 << 0x0A,
        OLEN_16         = 0x2 << 0x0A,
        OLEN_32         = 0x3 << 0x0A,

        IN_LSB          = 0x0 << 0x10,          /* input lst first      */
        IN_MSB          = 0x1 << 0x10,          /* input msb first      */

        OUT_LSB         = 0x0 << 0x11,          /* output lsb first     */
        OUT_MSB         = 0x1 << 0x11;          /* output msb first     */


